package com.trust.ywx.customer.service.manager.impl;

import com.trust.ywx.customer.common.enu.FirmErrEum;
import com.trust.ywx.customer.commons.UniqueIdUtils;
import com.trust.ywx.customer.dao.FirmExtendDao;
import com.trust.ywx.customer.dao.TrustFirmDao;
import com.trust.ywx.customer.dao.po.FirmInfo;
import com.trust.ywx.customer.common.model.manager.request.TrustFirmParam;
import com.trust.ywx.customer.common.model.manager.response.TrustFirmDto;
import com.trust.ywx.customer.common.exception.BizException;
import com.trust.ywx.customer.service.manager.FirmManagerService;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.TransactionCallback;
import org.springframework.transaction.support.TransactionCallbackWithoutResult;
import org.springframework.transaction.support.TransactionTemplate;

import javax.annotation.Resource;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * <pre></pre>
 *
 * @文件名称： FirmManagerServiceImpl.java
 * @包 路  径： com.trust.ywx.customer.service.manager.impl
 * @Copyright：北京数字医信责任有限公司 (C) 2022 *
 * @Description: 客户管理实现类
 * @Version: V1.0
 * @Author： hanzhenchao
 * @Date：2022/6/20 12:50
 * @Modify：
 */
@Service
public class FirmManagerServiceImpl implements FirmManagerService {
    @Resource
    TransactionTemplate transactionTemplate;
    @Resource
    TrustFirmDao trustFirmDao;
    @Resource
    FirmExtendDao firmExtendDao;

//    @Resource
//    private Config config;

    @Override
    public TrustFirmDto init(String clientId) {
        TrustFirmDto dto = new TrustFirmDto();
        dto.setClientId(UniqueIdUtils.generateClientId());
        dto.setAppSecret(UniqueIdUtils.generateAppSecret());
        // TODO 其他默认选项
        return dto;
    }

    @Override
    public boolean add(TrustFirmParam param) throws BizException {
        // 参数校验
        checkParam(param);
        // 业务校验
        checkAddBiz(param);
        // 执行插入
        return doAddTrust(param);
    }

    private void checkParam(TrustFirmParam param) {
        //TODO
    }

    @Override
    public boolean update(TrustFirmParam param) throws BizException {
        // 参数校验
        checkParam(param);
        // 业务校验
        checkUpdateBiz(param);
        // 执行插入
        return doUpdateTrust(param);
    }

    @Override
    public boolean delete(String clientId) {
        // 逻辑删除可信客户
        trustFirmDao.deleteByClientId(clientId);
        return true;
    }

    /**
     * 执行添加（包含事务）
     *
     * @param param
     * @return
     */
    private boolean doAddTrust(TrustFirmParam param) {
        transactionTemplate.execute(new TransactionCallbackWithoutResult() {
            @Override
            protected void doInTransactionWithoutResult(TransactionStatus transactionStatus) {
                // 插入firm_info表
                FirmInfo firmInfo = convertToFirmInfo(param, true);
                trustFirmDao.insert(firmInfo);

                // 插入firm_info_extend表
                Map<String, String> propMap = param.convertToMap();
                firmExtendDao.batchInsert(param.getClientId(), propMap);

            }
        });
        return true;
    }

    private FirmInfo convertToFirmInfo(TrustFirmParam param, boolean isAdd) {
        FirmInfo firmInfo = new FirmInfo();
        BeanUtils.copyProperties(param, firmInfo);
        if (isAdd) {
            firmInfo.setUniqueid(UniqueIdUtils.genrateUniqueId());
        }
        firmInfo.setStatus(param.getFirmStatus());
        // TODO 客户属性有几个属性在此表维护，以后要迁移至extend表。
        firmInfo.setSendSms(param.getSmsSend() == null ? "N" : (param.getSmsSend() == 1 ? "Y" : "N"));
        firmInfo.setStampType(param.getUserStampAuto() == null ? "0" : String.valueOf(param.getUserStampAuto()));
        return firmInfo;
    }


    /**
     * 校验业务-新建
     *
     * @param param
     */
    private void checkAddBiz(TrustFirmParam param) throws BizException {
        //clientId冲突校验
        FirmInfo firmInfo = trustFirmDao.getFrimByClientId(param.getClientId());
        if (null != firmInfo) {
            throw new BizException(FirmErrEum.CLIENTID_CHECK_ERR);
        }
        // clientOrgCode冲突校验
        List<FirmInfo> list = trustFirmDao.getListByOrgCode(param.getClientOrgCode());
        if (null != list && list.size() > 0) {
            throw new BizException(FirmErrEum.CLIENT_ORG_CODE_CHECK_ERR);
        }
        // firmName冲突校验
        list = trustFirmDao.getListByFirmName(param.getFirmName());
        if (null != list && list.size() > 0) {
            throw new BizException(FirmErrEum.FIRM_NAME_CHECK_ERR);
        }
    }

    /**
     * 校验业务-更新
     *
     * @param param
     */
    private void checkUpdateBiz(TrustFirmParam param) throws BizException {

        //clientId冲突校验
        FirmInfo firmInfo = trustFirmDao.getFrimByClientId(param.getClientId());
        if (null == firmInfo) {
            throw new BizException(FirmErrEum.FIRM_NOT_EXIST);
        }

        // clientOrgCode冲突校验
        List<FirmInfo> list = trustFirmDao.getListByOrgCode(param.getClientOrgCode());
        if (list != null && (list.size() > 1 || Objects.equals(param.getClientId(), list.get(0).getClientId()))) {
            throw new BizException(FirmErrEum.CLIENT_ORG_CODE_CHECK_ERR);
        }
        // firmName冲突校验
        list = trustFirmDao.getListByFirmName(param.getFirmName());
        if (list != null && (list.size() > 1 || Objects.equals(param.getClientId(), list.get(0).getClientId()))) {
            throw new BizException(FirmErrEum.FIRM_NAME_CHECK_ERR);
        }
    }


    private boolean doUpdateTrust(TrustFirmParam param) {
        int num = transactionTemplate.execute(new TransactionCallback<Integer>() {

            @Override
            public Integer doInTransaction(TransactionStatus transactionStatus) {
                // 插入firm_info表
                FirmInfo firmInfo = convertToFirmInfo(param, false);
                int num = trustFirmDao.update(firmInfo);

                // 插入firm_info_extend表
                Map<String, String> propMap = param.convertToMap();
                firmExtendDao.batchInsert(param.getClientId(), propMap);
                return num;
            }
        });
        return num == 1;
    }

    @Override
    public boolean addCaChannelFirm() {
        return false;
    }

    @Override
    public boolean addFirmWhite() {
        return false;
    }
}
